// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.IO;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.WebSitesExtensions.Models;

namespace Microsoft.WindowsAzure.WebSitesExtensions
{
    /// <summary>
    /// Operations for managing continuous WebJobs.
    /// </summary>
    public partial interface IContinuousWebJobOperations
    {
        /// <summary>
        /// Delete a continuous job.
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> DeleteAsync(string jobName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Get a continuous WebJob.
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The get continuous WebJob Operation Response.
        /// </returns>
        Task<ContinuousWebJobGetResponse> GetAsync(string jobName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Get the settings of a continuous WebJob.
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The continuous WebJob settings operation response.
        /// </returns>
        Task<ContinuousWebJobSettingsResponse> GetSettingsAsync(string jobName, CancellationToken cancellationToken);
        
        /// <summary>
        /// List the continuous web jobs.
        /// </summary>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The list of continuous WebJobs operation response.
        /// </returns>
        Task<ContinuousWebJobListResponse> ListAsync(CancellationToken cancellationToken);
        
        /// <summary>
        /// Set the settings of a continuous WebJob (will replace the current
        /// settings of the WebJob).
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='settings'>
        /// The continuous WebJob settings.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> SetSettingsAsync(string jobName, ContinuousWebJobSettingsUpdateParameters settings, CancellationToken cancellationToken);
        
        /// <summary>
        /// Start a continuous WebJob.
        /// </summary>
        /// <param name='jobName'>
        /// The WebJob name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> StartAsync(string jobName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Stop a continuous WebJob.
        /// </summary>
        /// <param name='jobName'>
        /// The WebJob name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> StopAsync(string jobName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Create or replace a continuous WebJob with a script file (.exe,
        /// .bat, .php, .js...).
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// The file name.
        /// </param>
        /// <param name='jobContent'>
        /// The file content.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> UploadFileAsync(string jobName, string fileName, Stream jobContent, CancellationToken cancellationToken);
        
        /// <summary>
        /// Create or replace a continuous WebJob with a zip file (containing
        /// the WebJob binaries).
        /// </summary>
        /// <param name='jobName'>
        /// The continuous WebJob name.
        /// </param>
        /// <param name='fileName'>
        /// The zip file name.
        /// </param>
        /// <param name='jobContent'>
        /// The zip file content.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        Task<AzureOperationResponse> UploadZipAsync(string jobName, string fileName, Stream jobContent, CancellationToken cancellationToken);
    }
}
